<?php
namespace PME\Controllers\Custo;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class CustoPlanejadoRealizadoController extends \Zeedhi\Framework\Controller\Report {
  protected $entityManager;
  protected $birtStrategy;
  protected $environment;
  protected $newCode;
  protected $connection;
  protected $lista;
  protected $ConsumoPlanejadoRealizadoController;

  public function __construct(ReportService $reportService, $entityManager, $birtStrategy, $environment, $newCode, $connection, $lista, $ConsumoPlanejadoRealizadoController) {
    $this->entityManager = $entityManager;
    $this->birtStrategy  = $birtStrategy;
    $this->environment   = $environment;
    $this->newCode       = $newCode;
    $this->connection    = $connection;
    $this->lista = $lista;
    $this->ConsumoPlanejadoRealizadoController = $ConsumoPlanejadoRealizadoController;
    
    parent::__construct($reportService);
  }

  protected $reportMapping = array (
    'PME01103' => 
    array (
      'strategy' => 'BIRT',
      'parameterMapping' => 
      array (
        "TAMACNIVEL"      => "P_NRTAMACNIVEL",
        "NIVEL"           => "P_NIVEL",
        "CDFILCENTPME"    => "P_CDFILCENTPME",
        "DTDE"            => "P_DTINICIAL",
        "DTATE"           => "P_DTFINAL",
        "LISTACARD"       => "P_LISTATIPOCARD", 
        "LISTASERV"       => "P_LISTASERVICO", 
        "CDTIPOCUSTOREAL" => "P_NRTIPOCUSTO_REAL",
        "ESTOCAVEIS"      => "P_IDESTOCAVEIS",
        "NRCONSPLAN"      => "P_NRCONSPLAN", 
        "NRLISTAFILI"     => "P_NRLISTAFILI", 
        "NAOESTOCAVEIS"   => "P_IDNESTOCAVEIS",
        "NMTIPOCUSTOPLAN" => "P_NMTIPOCUSTO",
        "AGRUPMASK"       => "P_MASCAGRUP",
        "DIFQTD"          => "IDDIFERENCA",
        "NRORG"           => "P_NRORG",
        "CDTIPOCUSTOPLAN" => "P_NRTIPOCUSTO",
        "NMTIPOCUSTOREAL" => "P_NMTIPOCUSTOREAL"
      )
    ),
    'PME01102' => 
    array (
      'strategy' => 'BIRT',
      'parameterMapping' => 
      array (
        "NRORG"           => "P_NRORG",
        "TAMACNIVEL"      => "P_NRTAMACNIVEL",
        "NIVEL"           => "P_NIVEL",
        "CDFILCENTPME"    => "P_CDFILCENTPME",
        "DTDE"            => "P_DTINICIAL",
        "DTATE"           => "P_DTFINAL",
        "LISTACARD"       => "P_LISTATIPOCARD", 
        "LISTASERV"       => "P_LISTASERVICO", 
        "CDTIPOCUSTOREAL" => "P_NRTIPOCUSTO_REAL",
        "ESTOCAVEIS"      => "P_IDESTOCAVEIS",
        "NRCONSPLAN"      => "P_NRCONSPLAN", 
        "NRLISTAFILI"     => "P_NRLISTAFILI", 
        "NAOESTOCAVEIS"   => "P_IDNESTOCAVEIS",
        "DTMESINIC"       => "P_DTMESINIC",
        "DTMESFINA"       => "P_DTMESFINA",  
        "AGRUPMASK"       => "P_MASCAGRUP",
        "NMTIPOCUSTOPLAN" => "P_NMTIPOCUSTO",
        "DIFQTD"          => "P_IDDIFERENCA",
        "CDTIPOCUSTOPLAN" => "P_NRTIPOCUSTO",
        "NMTIPOCUSTOREAL" => "P_NMTIPOCUSTOREAL"
      )
    )
  );
    
  public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
    $rows = $request->getRow();
    $nrorg = $this->environment->getNrOrgTrab();

    $rows['NRORG'] = $nrorg;
    $rows['NRCONSPLAN'] = $this->newCode->getCodeByTableName('CONSPLAN'.$rows['CDFILCENTPME'], $nrorg);
    
    $unidade = [];
    foreach($rows['UNIDADE'] as $row){
      $concatUnid = $row;
      array_push($unidade, $concatUnid);
    }
    
    if ($rows['GRUPFILI'] == NULL){
      if(empty($rows['UNIDADE'])){
        $unid = $this->ConsumoPlanejadoRealizadoController->getUnidades($rows);
        $rows['NRLISTAFILI'] = $this->lista->geraLista($unid, $nrorg);
      } else $rows['NRLISTAFILI'] = $this->lista->geraLista(join("_", $unidade), $nrorg);
    } else if ($rows['GRUPFILI'] !== NULL){
      if(empty($rows['UNIDADE'])){
        $unid = $this->ConsumoPlanejadoRealizadoController->getUnidadesGrupoServ($rows, $nrorg);  
        $rows['NRLISTAFILI'] = $this->lista->geraLista($unid, $nrorg);
      } else $rows['NRLISTAFILI'] = $this->lista->geraLista(join("_", $unidade), $nrorg); 
    }
    
    $rows['LISTACARD'] = $this->geraListaTipoCard($rows, $nrorg);
    $rows['LISTASERV'] = $this->geraListaServico($rows, $nrorg);

    $this->ConsumoPlanejadoRealizadoController->procedure($rows);
    
    $this->birtStrategy->setReportLocale($this->environment->getLanguage());
    $this->birtStrategy->setReportFormat(strtolower($rows['reportFormat']));
  }
  
  public function geraListaTipoCard($rows, $nrorg) {
    $nrlista = $this->newCode->getCodeByTableName('LISTA_TIPOCARD', $rows['NRORG']);

    $sql = "INSERT INTO LISTA_TIPOCARD (NRLISTA, CDFILCENTPME, CDTIPOCARD, NMTIPOCARD, NRORG) VALUES (:NRLISTA, :CDFILCENTPME, :CDTIPOCARD, :NMTIPOCARD, :NRORG)";
    $cardapio = $rows['TIPOCARD'];
    
    if(count($cardapio) == 0){
      $result = $this->getCardapio($rows, $nrorg);
      $cardapio = $result;
    }

    foreach ($cardapio as $card) {
      $stmt = $this->entityManager->getConnection()->prepare($sql);
      $stmt->execute(
        array(
          "NRLISTA"      => $nrlista,
          "CDFILCENTPME" => $rows['CDFILCENTPME'],
          "CDTIPOCARD"   => $card['CDTIPOCARD'],
          "NMTIPOCARD"   => $card['NMTIPOCARD'],
          "NRORG"        => $nrorg
        )
      );
    }
    return $nrlista;
  }
  
  public function geraListaServico($rows, $nrorg) {
    $nrlista = $this->newCode->getCodeByTableName('LISTA_SERVICO', $rows['NRORG']);
    
    $sql = "INSERT INTO LISTA_SERVICO (NRLISTA, CDFILIAL, CDSERVFILI, NMSERVICO, CDGRUPSERV, NRORG) VALUES (:NRLISTA, :CDFILIAL, :CDSERVFILI, :NMSERVICO, :CDGRUPSERV, :NRORG)";
    $servicosFiliais = $rows['SERVICO'];
    
    if(count($servicosFiliais) == 0){
      $result = $this->getUnidServ($rows);
      $servicosFiliais = $result;
    }
    
    foreach ($servicosFiliais as $servicoFilial) {
      $stmt = $this->entityManager->getConnection()->prepare($sql);
      $stmt->execute(
        array(
          "NRLISTA"    => $nrlista,
          "CDFILIAL"   => $servicoFilial['CDFILIAL'],
          "NRORG"      => $rows['NRORG'],
          "CDSERVFILI" => $servicoFilial['CDSERVFILI'],
          "NMSERVICO"  => $servicoFilial['NMSERVICO'],
          "CDGRUPSERV" => $servicoFilial['CDGRUPSERV']
        )
      );
    }
    return $nrlista;
  }
  
  public function getUnidServ($rows){
    $nrorg = $this->environment->getNrOrgTrab();
    
    $unidade = [];
    foreach($rows['UNIDADE'] as $row){
      $concatUnid = $row;
      array_push($unidade, $concatUnid);
    }
    
    if ($rows['GRUPFILI'] == NULL) {
      if (empty($rows['UNIDADE'])) $rows['UNIDADE'] = $this->ConsumoPlanejadoRealizadoController->getUnidades($rows);
      else  $rows['UNIDADE'] = is_array($rows['UNIDADE']) ? join("_", $unidade) : $rows['UNIDADE'];
    } else if ($rows['GRUPFILI'] !== NULL) {
      if (empty($rows['UNIDADE'])) $rows['UNIDADE'] = $this->ConsumoPlanejadoRealizadoController->getUnidadesGrupoServ($rows, $nrorg);  
      else $rows['UNIDADE'] = join("_", $unidade);
    }
    
    if (empty($rows['GRUPOSERVICO'])) $rows['GRUPOSERVICO'] = 'T';   
    if (empty($rows['CDGRUPFILI'])) $rows['CDGRUPFILI'] = 'T';
    
    $query = "SELECT DISTINCT S.CDGRUPSERV, S.CDFILIAL, S.CDSERVFILI, S.NMSERVICO, ( S.CDFILIAL || S.CDSERVFILI ) AS FILISERV, S.NRORG FROM SERVICO      S, FILGRUPFILI  F, FILIAL       FI WHERE ( S.CDFILIAL = F.CDFILIAL (+) ) AND ( FI.CDFILIAL = S.CDFILIAL ) AND ( ( INSTR(:CDFILIAL, S.CDFILIAL) <> 0 ) OR ( :CDFILIAL = 'T' ) ) AND ( ( INSTR(:CDGRUPSERV, S.CDGRUPSERV) <> 0 ) OR ( :CDGRUPSERV = 'T' ) ) AND ( ( INSTR(:CDGRUPFILI, F.CDGRUPFILI) <> 0 ) OR ( :CDGRUPFILI = 'T' ) ) AND ( FI.CDFILCENTPME = :CDFILCENTPME ) ORDER BY S.CDFILIAL, S.CDSERVFILI";
    $params = array(
      "CDFILCENTPME" => $rows["CDFILCENTPME"],
      "CDGRUPFILI"   => $rows['CDGRUPFILI'],
      "CDFILIAL"     => $rows['UNIDADE'],
      "CDGRUPSERV"   => $rows['GRUPOSERVICO']
    );
    $stmt = $this->connection->prepare($query);
	  $stmt->execute($params);
    $result = $stmt->fetchAll();
    
    return $result;
  }
  
  public function getCardapio($rows, $nrorg){
    $cdoperador = $this->environment->getCdOperador(); 
    $query = "SELECT CDTIPOCARD, NMTIPOCARD, (CDTIPOCARD || ' - ' || NMTIPOCARD) AS DESCRICAO,  IDTPCARDATIV, NRPASTATCARD, NRORG FROM TIPOCARD WHERE NRORG = :NRORG AND IDTPCARDATIV = 'S' AND CDFILCENTPME = :CDFILCENTPME AND (NRPASTATCARD = :NRPASTATCARD OR :NRPASTATCARD IS NULL) ORDER BY CDTIPOCARD";
    $params = array(
      "NRORG"        => $nrorg,
      "NRPASTATCARD" => NULL,
      "CDFILCENTPME" => $rows["CDFILCENTPME"]
    );
    $stmt = $this->connection->prepare($query);
	  $stmt->execute($params);
    $result = $stmt->fetchAll();
    
    return $result;
  }
}