import {
  Loader, Metadata, IEventParam, IDictionary, Config,
} from '@zeedhi/core';
import {
  Form, Select, Button, CheckboxMultiple,
} from '@zeedhi/common';
import { PmeMask } from '@/config';

export class Pme01700CustoRealGlobalPeriodoController {
  public reportController = Loader.getInstance('ReportController');

  public appController = Loader.getInstance('AppController');

  public utilities = Loader.getInstance('Utilities');
  
  public modalController = Loader.getInstance('ModalController');

  public defaultReport({ component }: IEventParam<Select>) {
    component.setValue(component.datasource.data[0]);
  }
  
  private impTipoProduto = {
    ESTOCAVEIS: 'S',
    NAOESTOCAVEIS: 'S',
  };
  
  private setParams(row: IDictionary) {
    const maskProd: string = (Config.masks.productMask as PmeMask).pattern;
    row.CDTIPOCUSTO = row.TIPOCUSTO.CDTIPOCUSTO;
    row.TAMACNIVEL = this.utilities.calcTamacNivel(row.NIVEL, 'produto');
    row.ESTOCAVEIS = this.impTipoProduto.ESTOCAVEIS;
    row.CDSERVFILI = row.SERVICO ? row.SERVICO : null;
    row.LISTASERV = '';
    row.DTMESINIC = row.DTDE;
    row.DTMESFINA = row.DTATE;
    row.NAOESTOCAVEIS = this.impTipoProduto.NAOESTOCAVEIS;
    row.TAMCODIGO = '';
    row.CDFILCENTPME = this.appController.getCdCentral();
    row.NMFILCENTPME = this.appController.getNmCentral();
    row.NMTIPOCUSTO = row.TIPOCUSTO.NMTIPOCUSTO;
    row.AGRUPMASK = row.NIVEL ? this.utilities.getReportMaskNivel(maskProd, row.NIVEL) : 0;
    row.REPORT_NAME = row.REPORT.REPORT_NAME;
    row.CDPICTPROD = this.utilities.formatReportMask(maskProd);
    row.CD_REPORT_CODE = row.REPORT.CD_REPORT_CODE;
  }
  
  public generateReport({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('FORM');
    const tipoProduto = Metadata.getInstance<CheckboxMultiple>('TIPOPRODUTO');

    if (this.reportController.formIsValid(form)) {
      const row: IDictionary = form.value;
      
      this.setParams(row);
      if (row.CDFILCENTPME == null) this.utilities.showAlert('alertError', 'pme::centralNaoInformada', 'error');
      if (tipoProduto.value.length === 0) this.utilities.showAlert('alertError', 'pme::tipoProdutoDeveInformar', 'error');
      else {
        row.reportFormat = component.name;
        this.utilities.activeLoadingButton(component);
        this.reportController.generateReport('/report/custoRealizadoGlobalPeriodo', row, row.CD_REPORT_CODE).finally(() => {
          this.utilities.disableLoadingButton(component);
        });
      }
    }
  }
  
  public setTipoProduto({ component }: IEventParam<CheckboxMultiple>) {
    if (component.value) {
      const items = component.value.join('_');
      if (items.includes('ESTOCAVEIS')) this.impTipoProduto.ESTOCAVEIS = 'S';
      else this.impTipoProduto.ESTOCAVEIS = 'N';

      if (items.includes('NAOESTOC')) this.impTipoProduto.NAOESTOCAVEIS = 'S';
      else this.impTipoProduto.NAOESTOCAVEIS = 'N';
    }
  }
}
