import {
  Http, IDictionary, IEventParam, Loader, Metadata, Config, Datasource,
} from '@zeedhi/core';
import {
  Button, Column, DialogService, Form, GridEditable, Select, SelectMultiple, TextInput,
} from '@zeedhi/common';
import { PmeMask } from '@/config';
  
export class Pme00200ServicoController {
  private utilities = Loader.getInstance('Utilities');
  
  public crudController = Loader.getInstance('CrudController');
  
  public appController = Loader.getInstance('AppController');

  public gridEditController = Loader.getInstance('GridEditableController');
  
  public modalController = Loader.getInstance('ModalController');

  public editable = false;

  public activeArv: object | undefined;

  public selectedServ?: string = '';

  public selectedNmServ?: string = '';
  
  public unidade?: string = '';
  
  public checkedRowsServ: IDictionary = {};
  
  public checkedRowsTipo: IDictionary = {};
  
  public currentRowCopy: IDictionary = {};
  
  public nrseqconfcard = 0;

  public showConfig = false;

  public showTipoPrat = false;

  public colsServ = 12;

  private isCentral = false;

  public get colsConfigServ() {
    return this.colsServ % 12;
  }
  
  public returnConditionsUnidade(nameFilter: string) {
    const unidSelecionada = Metadata.getInstance<Select>('UNIDADE');
    const conditions = [
      {
        name: nameFilter,
        operator: '=',
        value: unidSelecionada.value.CDFILIAL,
      },
    ];
    return conditions;
  }
  
  public fillCode({ component }: IEventParam<TextInput>) {
    component.value = component.value.padStart(5, '0');
  }
  
  public addRow({ component }: IDictionary) {
    this.editable = true;
    const grid: GridEditable = this.utilities.returnGrid(component) as GridEditable;

    if (!this.gridEditController.newRow) {
      if (grid.name === 'SERVICOS') {
        const item = { __is_new: true };
        grid.datasource.data.unshift(item);
      }
      this.gridEditController.newRow = true;
      grid.editing = true;
      if (!this.gridEditController.isInlineEditing) this.gridEditController.onInlineEdit({ component: grid });
    }
  }

  public cancelEditing({ component }: IDictionary) {
    const grid = this.utilities.returnGrid(component) as GridEditable;
    this.editable = false;
    if (grid.name === 'CONFIGTIPOPRAT') {
      this.gridEditController.defaultState(grid);
    } else this.gridEditController.cancelEditing({ component });
    
    this.resetTipoPrat();
  }
  
  public cancelEditingGridServ({ component }: IDictionary) {
    this.editable = false;
    const gridServ = this.utilities.returnGrid(component) as GridEditable;
    if (gridServ.editing) {
      this.gridEditController.newRow = false;
      gridServ.cancelEditedRows();
      this.gridEditController.onInlineEdit({ component: gridServ });
      gridServ.datasource.currentRow = {};
      gridServ.datasource.get();
    }
  }
  
  // seta primeira linha do grid servicos como sendo a current row
  private setCurrentRowGridServ() {
    const gridServ = Metadata.getInstance<GridEditable>('SERVICOS');
    gridServ.datasource.get().then(() => {
      const data = gridServ.datasource.data;
      if (data.length > 0) {
        gridServ.datasource.currentRow = data[0];
        this.setServ(gridServ.datasource.currentRow);
      }
    });
  }
  
  public reloadGridServicos() {
    this.setCurrentRowGridServ();
  }
  
  public returnColumn(component: GridEditable, nmColuna: string) {
    return component.columns.find((column) => column.name === nmColuna);
  }
  
  public clickRowGrid({ component }: IEventParam<Datasource>) {
    const gridServ = Metadata.getInstance<GridEditable>('SERVICOS');
    if (!gridServ.editing) {
      const row = component.currentRow;
      this.setServ(row);
      this.filterGridConfPrat();
    } else {
      this.showConfig = false;
      this.colsServ = 12;
    }
  }
  
  // Filtro
  public setFilterGridService({ component }: IEventParam<Select>) {
    const gridServ = Metadata.getInstance<GridEditable>('SERVICOS');
    const cdfilial = component.value;
    const addButton = Metadata.getInstance<Button>('addButton');
    const exportButton = Metadata.getInstance<Button>('exportButton');
    const reloadButton = Metadata.getInstance<Button>('reloadButton');
    this.isCentral = (cdfilial !== null && cdfilial.IDFILCENTPME === 'S');

    if ((cdfilial === null) || (cdfilial && cdfilial.IDFILCENTPME !== 'S' && cdfilial.CENTRAL === null)) {
      if (cdfilial && cdfilial.IDFILCENTPME !== 'S' && cdfilial.CENTRAL === null) this.utilities.showAlert('alertError', 'pme::unidadeSemCentralMerenda', 'error');
      gridServ.datasource.filter = [
        {
          name: 'P_CDFILIAL',
          operator: '=',
          value: '',
        },
        {
          name: 'P_NRORG',
          operator: '=',
          value: '',
        },
      ];
      gridServ.datasource.get().then(() => {
        exportButton.disabled = true;
        addButton.disabled = true;
        reloadButton.disabled = true;
      });
    } else {
      gridServ.datasource.filter = [
        {
          name: 'P_CDFILIAL',
          operator: '=',
          value: cdfilial.CDFILIAL,
        },
        {
          name: 'P_NRORG',
          operator: '=',
          value: localStorage.getItem('NRORG'),
        },
      ];
      gridServ.datasource.get().then(() => {
        if (cdfilial.CDFILIAL) {
          addButton.disabled = false;
          reloadButton.disabled = false;
          const data = gridServ.datasource.data;
          if (data.length === 0) gridServ.datasource.currentRow = {};

          this.setServ(gridServ.datasource.currentRow);

          if (cdfilial.IDFILCENTPME === 'S' && cdfilial.IDTPFILIALCENT === 'M') {
            exportButton.disabled = false;
            this.setDataModalExport(component);
          } else {
            exportButton.disabled = true;
          }
        } else {
          addButton.disabled = true;
          exportButton.disabled = true;
          reloadButton.disabled = true;
        }
      }).finally(() => {
        this.colsServ = 12;
        this.showConfig = false;
      });
    }
  }
  
  public setDataModalExport(component: Select) {
    this.unidade = component.formatter(component.value);
    const unidAgregada = Metadata.getInstance<Select>('UNIDADEAGREG');
    const serviceExport = Metadata.getInstance<Select>('SERVICO_EXPORT');
    unidAgregada.datasource.filter = this.returnConditionsUnidade('CDFILCENTPME');
    unidAgregada.datasource.get();
    serviceExport.datasource.filter = this.returnConditionsUnidade('CDFILIAL');
    serviceExport.datasource.get();
  }
  
  // Aba de Serviços(Grid)
  public filterServ({ component }: IEventParam<Select>) {
    const cdfilcentpme = this.appController.getCdCentral();
    const conditions = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: cdfilcentpme,
      },
    ];
    component.datasource.filter = conditions;
    component.datasource.get();
  }
  
  public filterProd({ component }: IEventParam<Select>) {
    component.datasource.get();
  }
  
  public saveService({ component }: IDictionary) {
    const form = Metadata.getInstance<Form>('formServico');
    const grid: GridEditable = this.utilities.returnGrid(component) as GridEditable;
    const rows: IDictionary = grid.getEditedRows();
    if (form.validate() && rows.length > 0) {
      const cdfilial = Metadata.getInstance<Select>('UNIDADE').value.CDFILIAL;
      let rowsInvalid = false;
      rows.forEach((e: IDictionary) => {
        if (this.validateFieldsHour(e.HRINICSERV, e.HRFINASERV, e.HRINICATEN)) {
          e.CDSERVFILI = e.CDSERVFILI ? e.CDSERVFILI.padStart(5, '0') : e.SERVICO.CDSERVPADR;
          e.HRINICATEN = e.HRINICATEN ? this.formatHrToSave(e.HRINICATEN) : null;
          e.HRINICSERV = e.HRINICSERV && e.HRFINASERV ? this.formatHrToSave(e.HRINICSERV) : null;
          e.HRFINASERV = e.HRFINASERV && e.HRINICSERV ? this.formatHrToSave(e.HRFINASERV) : null;
          e.CDGRUPSERV = e.CDGRUPSERV ? e.CDGRUPSERV : null;
          e.NMSERVICO = e.NMSERVICO ? e.NMSERVICO : e.SERVICO.NMSERVPADR;
          e.CDPRODUTO = e.CDPRODUTO ? e.CDPRODUTO : e.PRODUTO.CDPRODUTO;
          e.CDFILIAL = cdfilial;
        } else {
          rowsInvalid = true;
          // break;
        }
      });
      if (!rowsInvalid) {
        Http.post('/saveServ', { requestType: 'DataSet', rows }).then((response) => {
          const result = response.data.dataset.result;
          this.utilities.showAlert('alertError', result.message, result.type);
          this.gridEditController.defaultState(grid);
          this.gridEditController.reload({ component });
        }).catch(() => {
          this.utilities.showAlert('alertError', 'pme::impossivelAdicionar', 'error');
        });
      }
    }
  }
  
  public validateFieldsHour(hourIni: string, hourFin: string, hourAten: string) {
    let valid = true;
    if (hourIni && hourIni !== '' && hourFin && hourFin !== '') {
      const hrIni = hourIni.split(':');
      const secondsIni = (+hrIni[0]) * 60 * 60 + (+hrIni[1]) * 60;
      const hrFin = hourFin.split(':');
      const secondsFin = (+hrFin[0]) * 60 * 60 + (+hrFin[1]) * 60;
      
      if (!this.utilities.timeIsValid(hourIni) || !this.utilities.timeIsValid(hourFin) || !this.utilities.timeIsValid(hourAten)) {
        this.utilities.showAlert('alertHour', 'pme::horaInvalida', 'warning');
        valid = false;
      } else if (secondsIni > secondsFin) {
        this.utilities.showAlert('alertError', 'pme::horaInicialMaiorFinal', 'error');
        valid = false;
      }
    } else if (!this.utilities.timeIsValid(hourAten)) {
      this.utilities.showAlert('alertHour', 'pme::horaInvalida', 'warning');
      valid = false;
    }
    return valid;
  }
  
  public formatHrToSave(hour: string) {
    return hour.replace(':', '');
  }
  
  public confirmDeleteServ({ component }: IDictionary) {
    const grid: GridEditable = this.utilities.returnGrid(component) as GridEditable;
    this.checkedRowsServ = grid.selectedRows;
    if (this.checkedRowsServ.length === 0) this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    else {
      DialogService.show({
        name: 'pma00200_excluiServico',
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::desejaExcluirRegistros',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.deleteServ(this.checkedRowsServ, grid) },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }
  
  public deleteServ(checkedRows: IDictionary, grid: GridEditable) {
    Http.post('/deleteServ', { checkedRows }).then(() => {
      grid.cancelEditedRows();
      grid.datasource.get();
      grid.selectedRows = [];
      DialogService.hide();
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
    }).finally(() => {
      DialogService.hide();
    });
  }
  
  // Aba de Configuração
  
  public confirmDeleteConfig({ component }: IDictionary) {
    const grid: GridEditable = this.utilities.returnGrid(component) as GridEditable;
    this.checkedRowsTipo = grid.selectedRows;
    if (this.checkedRowsTipo.length === 0) this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    else {
      DialogService.show({
        name: 'pme00200_excluiConfiguracaoServ',
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::desejaExcluirRegistros',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.deleteRowsConfigPrat(this.checkedRowsTipo, grid) },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }
  
  public deleteRowsConfigPrat(checkedRows: IDictionary, grid: GridEditable) {
    Http.post('/deleteConfig', { checkedRows }).then(() => {
      grid.cancelEditedRows();
      grid.datasource.get();
      grid.selectedRows = [];
      this.getMaxSeq();
      this.utilities.showAlert('alertSuccess', 'pme::excluidoComSucesso', 'green');
      DialogService.hide();
    }).catch((error: string) => {
      this.utilities.showAlert('alertError', `pme::${error}`, 'error');
      DialogService.hide();
    });
  }
  
  public setServ(row: IDictionary) {
    if (row.CDSERVFILI) {
      this.selectedServ = row.CDSERVFILI;
      this.selectedNmServ = row.NMSERVICO ? `${row.CDSERVFILI} - ${row.NMSERVICO}` : row.SERVICO;

      if (this.isCentral) {
        this.showConfig = true;
        this.colsServ = 6;
        this.getMaxSeq();
      } else {
        this.showConfig = false;
        this.colsServ = 12;
      }
    } else {
      this.showConfig = false;
      this.colsServ = 12;
      this.selectedServ = '';
      this.selectedNmServ = '';
    }
  }

  public filterGridConfPrat() {
    const gridServ = Metadata.getInstance<GridEditable>('SERVICOS');
    const currentRow = gridServ.datasource.currentRow;
    const cdfilial = Metadata.getInstance<Select>('UNIDADE').value.CDFILIAL;
    const gridConf = Metadata.getInstance<GridEditable>('CONFIGTIPOPRAT');
    if (currentRow.CDSERVFILI) {
      const conditions = [
        {
          name: 'CDFILIAL',
          operator: '=',
          value: cdfilial,
        },
        {
          name: 'CDSERVFILI',
          operator: '=',
          value: this.selectedServ,
        },
      ];
      if (gridConf.editing || this.showTipoPrat) {
        this.showTipoPrat = false;
        this.gridEditController.defaultState(gridConf);
      }
      this.resetTipoPrat();

      gridConf.datasource.filter = conditions;
      gridConf.datasource.get();
    }
  }

  private resetTipoPrat() {
    const tipoPrat = Metadata.getInstance<Select>('TIPOPRAT');
    tipoPrat.setValue([]);
    this.showTipoPrat = false;
  }
  
  /* Pega o maior valor de NRSEQCNFCARD do banco para o serviço e aunidade selecionada
    Para não gerar KEY repetida e diferenciar as linhas do grid na hora de
    adicionar no grid de Configuração Padrão,pois a chave primária da tabela só
    é inserida na hora de salvar recebendo o número da ordem que foi digitado;
  */
  public async getMaxSeq() {
    const cdfilial = Metadata.getInstance<Select>('UNIDADE').value.CDFILIAL;
    const parametros = {
      CDFILIAL: cdfilial,
      CDSERVFILI: this.selectedServ,
    };
    Http.post('dadosGridConfigPrat', { requestType: 'Row', row: parametros }).then((response) => {
      const rowsJaExistentes: IDictionary = response.data ? response.data.dataset.tipoprat : [];
      if (rowsJaExistentes.length > 0) {
        const nrseqExistentes = rowsJaExistentes.map((row: IDictionary) => row.NRSEQCNFCARD);
        this.nrseqconfcard = Math.max(...nrseqExistentes);
      }
    });
  }

  public addConf() {
    const configTipoPrat: GridEditable = Metadata.getInstance('CONFIGTIPOPRAT');
    if (this.utilities.centralIsValid() && !configTipoPrat.editing) {
      this.showTipoPrat = true;
      this.gridEditController.newRow = true;
      this.gridEditController.onInlineEdit({ component: configTipoPrat });
    }
  }
  
  /* Salva grid de configuração de prato;
    Sempre irá deletar todas as rows e inserir novamente(mesmo em caso de edição),
    pois precisa fazer o controle do parâmetro de paginação;
    Regra para salvar o NRPAGCNFCARD(paginação): A cada duas inserções onde a coluna IDIMPCNFCARD for igual a 'S'
    aumenta a paginação em 1.
  */
  public saveConfigPrat({ component }: IDictionary) {
    const form = Metadata.getInstance<Form>('formConfig');
    if (form.validate()) {
      const cdfilial = Metadata.getInstance<Select>('UNIDADE').value.CDFILIAL;
      const tipoPrat = Metadata.getInstance<Select>('TIPOPRAT');

      const grid: GridEditable = this.utilities.returnGrid(component) as GridEditable;

      let rows: IDictionary = grid.getEditedRows();
      
      if (this.showTipoPrat) {
        const addRows = tipoPrat.value;
        addRows.forEach((e: IDictionary) => {
          e.__is_new = true;
          this.nrseqconfcard += 1;
          e.NRORDCNFCARD = this.nrseqconfcard;
          e.NRSEQCNFCARD = e.NRORDCNFCARD;
          e.IDIMPCNFCARD = 'S';
          e.IDOPCCNFCARD = 'N';
          e.NEW = 'S';
        });
        rows = rows.concat(addRows);
      }

      const parametros = {
        CDFILIAL: cdfilial,
        CDSERVFILI: this.selectedServ,
      };
      if (rows.length > 0) {
        Http.post('dadosGridConfigPrat', { requestType: 'Row', row: parametros }).then((response) => {
          const rowsJaExistentes: IDictionary = response.data ? response.data.dataset.tipoprat : [];
          const rowsTotal = rows.concat(rowsJaExistentes);
          const uniqueData = rowsTotal.map((row: IDictionary) => row.NRORDCNFCARD) // Usa uma key unica do objeto para utilizar na comparação
            .map((cdprato: string, index: number, final: string) => final.indexOf(cdprato) === index && index) // Pega o index de cada elemento do array.
            .filter((num: number) => rowsTotal[num])
            .map((position: number) => rowsTotal[position]);
          let inRow = 1;
          let inPag = 1;
          uniqueData.forEach((e: IDictionary) => {
            if (!e.IDIMPCNFCARD || e.IDIMPCNFCARD === 'N') {
              e.NRPAGCNFCARD = 0;
            } else if (e.IDIMPCNFCARD === 'S') {
              if (inRow === 1) {
                e.NRPAGCNFCARD = inPag;
                inRow = 2;
              } else {
                e.NRPAGCNFCARD = inPag;
                inRow = 1;
                inPag += 1;
              }
            }
            e.CDFILIAL = cdfilial;
            e.CDSERVFILI = this.selectedServ;
            e.NRSEQCNFCARD = e.NRORDCNFCARD;
          });
          Http.post('/saveConfigPratServ', { requestType: 'Row', row: uniqueData }).then((callback) => {
            if (callback) {
              this.editable = false;
              this.utilities.showAlert('alertSuccess', 'pme::salvoComSucesso', 'green');
              this.gridEditController.defaultState(grid);
            }

            tipoPrat.setValue([]);
            this.showTipoPrat = false;
          }).catch(() => {
            this.utilities.showAlert('alertError', 'pme::impossivelAdicionar', 'error');
          });
        });
      }
    }
  }
  
  // Exportar Serviços
  public openExportModal() {
    this.modalController.openModal();
  }
  
  public createModal() {
    this.modalController.createModal('/modals/formExportService');
  }

  public destroyModal() {
    this.modalController.destroyModal();
  }
  
  public setFilterFieldsExport({ component }: IEventParam<SelectMultiple>) {
    let filterName = 'CDFILCENTPME';
    if (component.name === 'SERVICO_EXPORT') filterName = 'CDFILIAL';
    component.datasource.filter = this.returnConditionsUnidade(filterName);
    // component.datasource.get();
  }
  
  public exportService({ component }: IEventParam<Button>) {
    const formExport = Metadata.getInstance<Form>('formExportService');
    if (formExport.validate()) {
      const row = {
        UNIDADE_EXPORT: Metadata.getInstance<Select>('UNIDADE').value.CDFILIAL,
        UNIDADEAGREG: formExport.value.UNIDADEAGREG,
        SERVICO_EXPORT: formExport.value.SERVICO_EXPORT,
      };
      row.UNIDADEAGREG = row.UNIDADEAGREG.length > 0 ? row.UNIDADEAGREG.join('_') : 'T';
      row.SERVICO_EXPORT = row.SERVICO_EXPORT.length > 0 ? row.SERVICO_EXPORT.join('_') : 'T';
      this.utilities.activeLoadingButton(component);
      Http.post('/exportService', { requestType: 'Row', row }).then((response) => {
        if (response.data.dataset.success) {
          this.utilities.showAlert('alertSuccess', 'pme::exportadoComSucesso', 'green');
          this.modalController.closeModal();
        } else {
          this.utilities.showAlert('alertError', 'pme::erroExportacao', 'red');
        }
      }).finally(() => {
        this.utilities.disableLoadingButton(component);
      });
    }
  }
  
  public produtoComponentProps = {
    component: 'ZdSelect',
    dataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    formatterDataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    dataValueOut: [
      {
        column: 'CDARVPROD',
        columnOnGrid: 'CDARVPROD',
      },
      {
        column: 'NMPRODUTO',
        columnOnGrid: 'NMPRODUTO',
      },
    ],
    dataValue: 'CDPRODUTO',
    dense: true,
    datasource: {
      uniqueKey: 'CDPRODUTO',
      route: '/prodUltimoNivel',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
    },
    validations: {
      required: {
        message: 'pme::campoObrigatorio',
      },
    },
  };
  
  public returnDadosPadraoServ({ component }: IEventParam<GridEditable>) {
    const row = component.datasource.currentRow;

    Http.post('/returnDadosPadraoServ', { requestType: 'Row', row }).then((response) => {
      const result = response.data.dataset.result;
      const servico = this.returnColumn(component, 'SERVICO') as Column;
      const cdservfili = this.returnColumn(component, 'CDSERVFILI') as Column;
      const nmservicoinput = this.returnColumn(component, 'NMSERVICO') as Column;
      if (result.length > 0) {
        servico.isVisible = true;
        cdservfili.isVisible = false;
        nmservicoinput.isVisible = false;
      } else {
        servico.isVisible = false;
        cdservfili.isVisible = true;
        nmservicoinput.isVisible = true;
      }
    });
  }

  public closeConfig() {
    const configTipoPrat: GridEditable = Metadata.getInstance('CONFIGTIPOPRAT');
    this.gridEditController.cancelEditing({ component: configTipoPrat });
    
    this.showTipoPrat = false;
    this.showConfig = false;
    this.colsServ = 12;
  }

  public reloadConfig({ component }: IDictionary) {
    const gridConfig: GridEditable = this.utilities.returnGrid(component) as GridEditable;
    this.gridEditController.reload({ component });
    this.resetTipoPrat();
    this.gridEditController.newRow = false;
    this.gridEditController.onInlineEdit({ component: gridConfig });
  }
}
