import {
  Loader, Metadata, IEventParam, IDictionary, Config,
} from '@zeedhi/core';
import {
  Select, GridEditable, Form, Button,
} from '@zeedhi/common';
import { PmeMask } from '@/config';
  
export class Pme02400ClientesPorServicoController {
  public appController = Loader.getInstance('AppController');

  public crudController = Loader.getInstance('CrudController');

  public gridEditController = Loader.getInstance('GridEditableController');

  public utilities = Loader.getInstance('Utilities');
  
  public clearFields() {
    const tpCardServ = Metadata.getInstance<Select>('TPCARDAPIOPORSERVICO');
    tpCardServ.setValue('');
  }
  
  public setTpCardServ({ component }: IEventParam<Select>) {
    const cdfilial = Metadata.getInstance<Select>('CDFILIAL');
    const filter = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: cdfilial.value ? cdfilial.value : null,
      },
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public reloadTelaClientesPorServico() {
    const formFiltro: Form = Metadata.getInstance<Form>('formFilter');
    if (formFiltro.validate()) {
      const gridClientesPorServ = Metadata.getInstance<GridEditable>('gridClientes');
      gridClientesPorServ.datasource.get();
    }
  }
  
  public setClientePorServico() {
    const addButton = Metadata.getInstance<Button>('addButton');
    const cdfilial = Metadata.getInstance<Select>('CDFILIAL');
    const tpCardServ = Metadata.getInstance<Select>('TPCARDAPIOPORSERVICO');
    const clientes = Metadata.getInstance<GridEditable>('gridClientes');
    
    if (!cdfilial.value || !tpCardServ.value) {
      clientes.datasource.data = [];
      addButton.disabled = true;
    } else {
      const valueTpCard: IDictionary = tpCardServ.value;
      const filter = [
        {
          name: 'CDFILIAL',
          operator: '=',
          value: cdfilial.value ? cdfilial.value : null,
        },
        {
          name: 'CDFILCENTPME',
          operator: '=',
          value: this.appController.getCdCentral(),
        },
        {
          name: 'CDTIPOCARD',
          operator: '=',
          value: valueTpCard.CDTIPOCARD ? valueTpCard.CDTIPOCARD : '',
        },
        {
          name: 'CDSERVFILI',
          operator: '=',
          value: valueTpCard.CDSERVFILI ? valueTpCard.CDSERVFILI : '',
        },
      ];
      clientes.datasource.filter = filter;
      addButton.disabled = true;
      clientes.datasource.get().then(() => {
        addButton.disabled = false;
      });
    }
  }

  public saveCliServ({ component }: IDictionary) {
    const grid = this.utilities.returnGrid(component) as GridEditable;
    const form = Metadata.getInstance<Form>('formFilter');
    const formClientes = Metadata.getInstance<Form>('formClientes');
    const gridClientes: GridEditable = Metadata.getInstance('gridClientes');
    const rows: IDictionary = gridClientes.getEditedRows();
    if (form.validate() && formClientes.validate() && rows.length > 0) {
      const unidade = form.value.CDFILIAL;
      const tpCardServ = form.value.TPCARDAPIOPORSERVICO;
      rows.forEach((e: IDictionary) => {
        e.CDFILCENTPME = this.appController.getCdCentral();
        e.CDFILIAL = unidade;
        e.CDSERVFILI = tpCardServ.CDSERVFILI;
        e.CDTIPOCARD = tpCardServ.CDTIPOCARD;
        e.CDCLIENTE = e.NRINSJURCLIE.CDCLIENTE;
      });

      this.crudController.saveRows(rows, 'servcliepme').then(() => {
        this.gridEditController.defaultState(grid);
        this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `pme::${error}`, 'error');
      });
    }
  }

  public addRow({ component }: IDictionary) {
    const form = Metadata.getInstance<Form>('formFilter');
    if (form.validate()) {
      this.gridEditController.addRow({ component });
    }
  }
  
  public clienteComponentProps = {
    component: 'ZdSelect',
    dataText: [
      {
        name: 'NRINSJURCLIE',
        mask: (Config.masks.cnpjMask as PmeMask).pattern,
      },
      'NMRAZSOCCLIE',
    ],
    formatterDataText: [
      {
        name: 'NRINSJURCLIE',
        mask: (Config.masks.cnpjMask as PmeMask).pattern,
      },
      'NMRAZSOCCLIE',
    ],
    dataValue: 'NRINSJURCLIE',
    dense: true,
    returnObject: true,
    datasource: {
      uniqueKey: 'NRINSJURCLIE',
      route: '/cliente',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
    },
    validations: {
      required: {
        message: 'pme::campoObrigatorio',
      },
    },
  };
}
